@@ -7,22 +7,21 @@ or linked to the appropriate data
 '''
 import struct
 
-from pymodbus.constants import ModbusStatus
+from pymodbus.constants import ModbusStatus, ModbusPlusOperation
 from pymodbus.pdu import ModbusRequest
 from pymodbus.pdu import ModbusResponse
-from pymodbus.pdu import ModbusExceptions as merror
 from pymodbus.device import ModbusControlBlock
 from pymodbus.exceptions import NotImplementedException
 from pymodbus.utilities import pack_bitstring
 
 _MCB = ModbusControlBlock()
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Function Codes Base Classes
 # diagnostic 08, 00-18,20
 #---------------------------------------------------------------------------#
-# TODO Implement subfunction 19 (Return IOP Overrun Count)
-# TODO Implement subfunction 21 (Get/Clear Modbus Plus statistics)
+# TODO Make sure all the data is decoded from the response
 #---------------------------------------------------------------------------#
 class DiagnosticStatusRequest(ModbusRequest):
     '''
@@ -63,6 +62,7 @@ class DiagnosticStatusRequest(ModbusRequest):
         '''
         self.sub_function_code, self.message = struct.unpack('>HH', data)
 
+
 class DiagnosticStatusResponse(ModbusResponse):
     '''
     This is a base class for all of the diagnostic response functions
@@ -106,6 +106,7 @@ class DiagnosticStatusResponse(ModbusResponse):
         '''
         self.sub_function_code, self.message = struct.unpack('>HH', data)
 
+
 class DiagnosticStatusSimpleRequest(DiagnosticStatusRequest):
     '''
     A large majority of the diagnostic functions are simple
@@ -133,6 +134,7 @@ class DiagnosticStatusSimpleRequest(DiagnosticStatusRequest):
         ''' Base function to raise if not implemented '''
         raise NotImplementedException("Diagnostic Message Has No Execute Method")
 
+
 class DiagnosticStatusSimpleResponse(DiagnosticStatusResponse):
     '''
     A large majority of the diagnostic functions are simple
@@ -141,7 +143,7 @@ class DiagnosticStatusSimpleResponse(DiagnosticStatusResponse):
     2 bytes of data.
     '''
 
-    def __init__(self, data):
+    def __init__(self, data=0x0000):
         ''' General initializer for a simple diagnostic response
 
         :param data: The resulting data to return to the client
@@ -149,6 +151,7 @@ class DiagnosticStatusSimpleResponse(DiagnosticStatusResponse):
         DiagnosticStatusResponse.__init__(self)
         self.message = data
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 00
 #---------------------------------------------------------------------------#
@@ -177,6 +180,7 @@ class ReturnQueryDataRequest(DiagnosticStatusRequest):
         '''
         return ReturnQueryDataResponse(self.message)
 
+
 class ReturnQueryDataResponse(DiagnosticStatusResponse):
     '''
     The data passed in the request data field is to be returned (looped back)
@@ -195,6 +199,7 @@ class ReturnQueryDataResponse(DiagnosticStatusResponse):
             self.message = message
         else: self.message = [message]
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 01
 #---------------------------------------------------------------------------#
@@ -227,6 +232,7 @@ class RestartCommunicationsOptionRequest(DiagnosticStatusRequest):
         #if _MCB.ListenOnly:
         return RestartCommunicationsOptionResponse(self.message)
 
+
 class RestartCommunicationsOptionResponse(DiagnosticStatusResponse):
     '''
     The remote device serial line port must be initialized and restarted, and
@@ -248,6 +254,7 @@ class RestartCommunicationsOptionResponse(DiagnosticStatusResponse):
             self.message   = [ModbusStatus.On]
         else: self.message = [ModbusStatus.Off]
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 02
 #---------------------------------------------------------------------------#
@@ -267,6 +274,7 @@ class ReturnDiagnosticRegisterRequest(DiagnosticStatusSimpleRequest):
         register = pack_bitstring(_MCB.getDiagnosticRegister())
         return ReturnDiagnosticRegisterResponse(register)
 
+
 class ReturnDiagnosticRegisterResponse(DiagnosticStatusSimpleResponse):
     '''
     The contents of the remote device's 16-bit diagnostic register are
@@ -274,6 +282,7 @@ class ReturnDiagnosticRegisterResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x0002
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 03
 #---------------------------------------------------------------------------#
@@ -295,6 +304,7 @@ class ChangeAsciiInputDelimiterRequest(DiagnosticStatusSimpleRequest):
         _MCB.Delimiter = char
         return ChangeAsciiInputDelimiterResponse(self.message)
 
+
 class ChangeAsciiInputDelimiterResponse(DiagnosticStatusSimpleResponse):
     '''
     The character 'CHAR' passed in the request data field becomes the end of
@@ -304,6 +314,7 @@ class ChangeAsciiInputDelimiterResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x0003
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 04
 #---------------------------------------------------------------------------#
@@ -324,6 +335,7 @@ class ForceListenOnlyModeRequest(DiagnosticStatusSimpleRequest):
         _MCB.ListenOnly = True
         return ForceListenOnlyModeResponse()
 
+
 class ForceListenOnlyModeResponse(DiagnosticStatusResponse):
     '''
     Forces the addressed remote device to its Listen Only Mode for MODBUS
@@ -334,6 +346,7 @@ class ForceListenOnlyModeResponse(DiagnosticStatusResponse):
     This does not send a response
     '''
     sub_function_code = 0x0004
+    should_respond    = False
 
     def __init__(self):
         ''' Initializer to block a return response
@@ -341,6 +354,7 @@ class ForceListenOnlyModeResponse(DiagnosticStatusResponse):
         DiagnosticStatusResponse.__init__(self)
         self.message = []
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 10
 #---------------------------------------------------------------------------#
@@ -359,6 +373,7 @@ class ClearCountersRequest(DiagnosticStatusSimpleRequest):
         _MCB.reset()
         return ClearCountersResponse(self.message)
 
+
 class ClearCountersResponse(DiagnosticStatusSimpleResponse):
     '''
     The goal is to clear ll counters and the diagnostic register.
@@ -366,6 +381,7 @@ class ClearCountersResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000A
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 11
 #---------------------------------------------------------------------------#
@@ -385,6 +401,7 @@ class ReturnBusMessageCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.BusMessage
         return ReturnBusMessageCountResponse(count)
 
+
 class ReturnBusMessageCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages that the
@@ -393,6 +410,7 @@ class ReturnBusMessageCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000B
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 12
 #---------------------------------------------------------------------------#
@@ -412,6 +430,7 @@ class ReturnBusCommunicationErrorCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.BusCommunicationError
         return ReturnBusCommunicationErrorCountResponse(count)
 
+
 class ReturnBusCommunicationErrorCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of CRC errors encountered
@@ -420,6 +439,7 @@ class ReturnBusCommunicationErrorCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000C
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 13
 #---------------------------------------------------------------------------#
@@ -439,6 +459,7 @@ class ReturnBusExceptionErrorCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.BusExceptionError
         return ReturnBusExceptionErrorCountResponse(count)
 
+
 class ReturnBusExceptionErrorCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of modbus exception
@@ -447,6 +468,7 @@ class ReturnBusExceptionErrorCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000D
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 14
 #---------------------------------------------------------------------------#
@@ -466,6 +488,7 @@ class ReturnSlaveMessageCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.SlaveMessage
         return ReturnSlaveMessageCountResponse(count)
 
+
 class ReturnSlaveMessageCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages addressed to the
@@ -474,6 +497,7 @@ class ReturnSlaveMessageCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000E
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 15
 #---------------------------------------------------------------------------#
@@ -493,6 +517,7 @@ class ReturnSlaveNoResponseCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.SlaveNoResponse
         return ReturnSlaveNoReponseCountResponse(count)
 
+
 class ReturnSlaveNoReponseCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages addressed to the
@@ -501,6 +526,7 @@ class ReturnSlaveNoReponseCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x000F
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 16
 #---------------------------------------------------------------------------#
@@ -521,6 +547,7 @@ class ReturnSlaveNAKCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.SlaveNAK
         return ReturnSlaveNAKCountResponse(count)
 
+
 class ReturnSlaveNAKCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages addressed to the
@@ -530,6 +557,7 @@ class ReturnSlaveNAKCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x0010
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 17
 #---------------------------------------------------------------------------#
@@ -549,6 +577,7 @@ class ReturnSlaveBusyCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.SlaveBusy
         return ReturnSlaveBusyCountResponse(count)
 
+
 class ReturnSlaveBusyCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages addressed to the
@@ -557,6 +586,7 @@ class ReturnSlaveBusyCountResponse(DiagnosticStatusSimpleResponse):
     '''
     sub_function_code = 0x0011
 
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 18
 #---------------------------------------------------------------------------#
@@ -578,6 +608,7 @@ class ReturnSlaveBusCharacterOverrunCountRequest(DiagnosticStatusSimpleRequest):
         count = _MCB.Counter.BusCharacterOverrun
         return ReturnSlaveBusCharacterOverrunCountResponse(count)
 
+
 class ReturnSlaveBusCharacterOverrunCountResponse(DiagnosticStatusSimpleResponse):
     '''
     The response data field returns the quantity of messages addressed to the
@@ -588,13 +619,44 @@ class ReturnSlaveBusCharacterOverrunCountResponse(DiagnosticStatusSimpleResponse
     '''
     sub_function_code = 0x0012
 
+
+#---------------------------------------------------------------------------#
+# Diagnostic Sub Code 19
+#---------------------------------------------------------------------------#
+class ReturnIopOverrunCountRequest(DiagnosticStatusSimpleRequest):
+    '''
+    An IOP overrun is caused by data characters arriving at the port
+    faster than they can be stored, or by the loss of a character due
+    to a hardware malfunction.  This function is specific to the 884.
+    '''
+    sub_function_code = 0x0013
+
+    def execute(self, *args):
+        ''' Execute the diagnostic request on the given device
+
+        :returns: The initialized response message
+        '''
+        count = _MCB.Counter.BusCharacterOverrun
+        return ReturnIopOverrunCountResponse(count)
+
+
+class ReturnIopOverrunCountResponse(DiagnosticStatusSimpleResponse):
+    '''
+    The response data field returns the quantity of messages
+    addressed to the slave that it could not handle due to an 884
+    IOP overrun condition, since its last restart, clear counters
+    operation, or power-up.
+    '''
+    sub_function_code = 0x0013
+
+
 #---------------------------------------------------------------------------#
 # Diagnostic Sub Code 20
 #---------------------------------------------------------------------------#
 class ClearOverrunCountRequest(DiagnosticStatusSimpleRequest):
     '''
     Clears the overrun error counter and reset the error flag
-    
+
     An error flag should be cleared, but nothing else in the
     specification mentions is, so it is ignored.
     '''
@@ -608,16 +670,57 @@ class ClearOverrunCountRequest(DiagnosticStatusSimpleRequest):
         _MCB.Counter.BusCharacterOverrun = 0x0000
         return ClearOverrunCountResponse(self.message)
 
+
 class ClearOverrunCountResponse(DiagnosticStatusSimpleResponse):
     '''
     Clears the overrun error counter and reset the error flag
     '''
     sub_function_code = 0x0014
 
-#---------------------------------------------------------------------------# 
+
+#---------------------------------------------------------------------------#
+# Diagnostic Sub Code 21
+#---------------------------------------------------------------------------#
+class GetClearModbusPlusRequest(DiagnosticStatusSimpleRequest):
+    '''
+    In addition to the Function code (08) and Subfunction code
+    (00 15 hex) in the query, a two-byte Operation field is used
+    to specify either a 'Get Statistics' or a 'Clear Statistics'
+    operation.  The two operations are exclusive - the 'Get'
+    operation cannot clear the statistics, and the 'Clear'
+    operation does not return statistics prior to clearing
+    them. Statistics are also cleared on power-up of the slave
+    device.
+    '''
+    sub_function_code = 0x0015
+
+    def execute(self, *args):
+        ''' Execute the diagnostic request on the given device
+
+        :returns: The initialized response message
+        '''
+        message = None # the clear operation does not return info
+        if self.message == ModbusPlusOperation.ClearStatistics:
+            _MCB.Plus.reset()
+        else: message = _MCB.Plus.encode()
+        return GetClearModbusPlusResponse(message)
+
+
+class GetClearModbusPlusResponse(DiagnosticStatusSimpleResponse):
+    '''
+    Returns a series of 54 16-bit words (108 bytes) in the data field
+    of the response (this function differs from the usual two-byte
+    length of the data field). The data contains the statistics for
+    the Modbus Plus peer processor in the slave device.
+    '''
+    sub_function_code = 0x0015
+
+
+#---------------------------------------------------------------------------#
 # Exported symbols
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 __all__ = [
+    "DiagnosticStatusRequest", "DiagnosticStatusResponse",
     "ReturnQueryDataRequest", "ReturnQueryDataResponse",
     "RestartCommunicationsOptionRequest", "RestartCommunicationsOptionResponse",
     "ReturnDiagnosticRegisterRequest", "ReturnDiagnosticRegisterResponse",
@@ -632,5 +735,7 @@ __all__ = [
     "ReturnSlaveNAKCountRequest", "ReturnSlaveNAKCountResponse",
     "ReturnSlaveBusyCountRequest", "ReturnSlaveBusyCountResponse",
     "ReturnSlaveBusCharacterOverrunCountRequest", "ReturnSlaveBusCharacterOverrunCountResponse",
+    "ReturnIopOverrunCountRequest", "ReturnIopOverrunCountResponse",
     "ClearOverrunCountRequest", "ClearOverrunCountResponse",
+    "GetClearModbusPlusRequest", "GetClearModbusPlusResponse",
 ]
