@@ -8,10 +8,10 @@ from pymodbus.constants import ModbusStatus
 from pymodbus.pdu import ModbusRequest
 from pymodbus.pdu import ModbusResponse
 from pymodbus.device import ModbusControlBlock
-from pymodbus.exceptions import *
 
 _MCB = ModbusControlBlock()
 
+
 #---------------------------------------------------------------------------#
 # TODO Make these only work on serial
 #---------------------------------------------------------------------------#
@@ -57,6 +57,7 @@ class ReadExceptionStatusRequest(ModbusRequest):
         '''
         return "ReadExceptionStatusRequest(%d)" % (self.function_code)
 
+
 class ReadExceptionStatusResponse(ModbusResponse):
     '''
     The normal response contains the status of the eight Exception Status
@@ -68,7 +69,7 @@ class ReadExceptionStatusResponse(ModbusResponse):
     function_code = 0x07
     _rtu_frame_size = 5
 
-    def __init__(self, status):
+    def __init__(self, status=0x00):
         ''' Initializes a new instance
 
         :param status: The status response to report
@@ -101,24 +102,26 @@ class ReadExceptionStatusResponse(ModbusResponse):
 # Encapsulate interface transport 43, 14
 # CANopen general reference 43, 13
 
+
 #---------------------------------------------------------------------------#
 # TODO Make these only work on serial
 #---------------------------------------------------------------------------#
 class GetCommEventCounterRequest(ModbusRequest):
     '''
-    This function code is used to get a status word and an event count from the
-    remote device's communication event counter. 
+    This function code is used to get a status word and an event count from
+    the remote device's communication event counter.
 
-    By fetching the current count before and after a series of messages, a client
-    can determine whether the messages were handled normally by the remote device. 
+    By fetching the current count before and after a series of messages, a
+    client can determine whether the messages were handled normally by the
+    remote device.
 
-    The device's event counter is incremented once  for each successful message
-    completion. It is not incremented for exception responses, poll commands,
-    or fetch event counter commands. 
+    The device's event counter is incremented once  for each successful
+    message completion. It is not incremented for exception responses,
+    poll commands, or fetch event counter commands.
 
-    The event counter can be reset by means of the Diagnostics function (code 08),
-    with a subfunction of Restart Communications Option (code 00 01) or
-    Clear Counters and Diagnostic Register (code 00 0A).
+    The event counter can be reset by means of the Diagnostics function
+    (code 08), with a subfunction of Restart Communications Option
+    (code 00 01) or Clear Counters and Diagnostic Register (code 00 0A).
     '''
     function_code = 0x0b
     _rtu_frame_size = 4
@@ -155,32 +158,34 @@ class GetCommEventCounterRequest(ModbusRequest):
         '''
         return "GetCommEventCounterRequest(%d)" % (self.function_code)
 
+
 class GetCommEventCounterResponse(ModbusResponse):
     '''
     The normal response contains a two-byte status word, and a two-byte
     event count. The status word will be all ones (FF FF hex) if a
-    previously-issued program command is still being processed by the remote
-    device (a busy condition exists). Otherwise, the status word will be 
-    all zeros.
+    previously-issued program command is still being processed by the
+    remote device (a busy condition exists). Otherwise, the status word
+    will be all zeros.
     '''
     function_code = 0x0b
     _rtu_frame_size = 8
 
-    def __init__(self, count):
+    def __init__(self, count=0x0000):
         ''' Initializes a new instance
 
         :param count: The current event counter value
         '''
         ModbusResponse.__init__(self)
         self.count = count
-        self.status = True # this means we are ready, not waiting
+        self.status = True  # this means we are ready, not waiting
 
     def encode(self):
         ''' Encodes the response
 
         :returns: The byte encoded message
         '''
-        ready = ModbusStatus.Ready if self.status else ModbusStatus.Waiting
+        if self.status: ready = ModbusStatus.Ready
+        else: ready = ModbusStatus.Waiting
         return struct.pack('>HH', ready, self.count)
 
     def decode(self, data):
@@ -199,27 +204,29 @@ class GetCommEventCounterResponse(ModbusResponse):
         arguments = (self.function_code, self.count, self.status)
         return "GetCommEventCounterResponse(%d, %d, %d)" % arguments
 
+
 #---------------------------------------------------------------------------#
 # TODO Make these only work on serial
 #---------------------------------------------------------------------------#
 class GetCommEventLogRequest(ModbusRequest):
     '''
-    This function code is used to get a status word, event count, message count,
-    and a field of event bytes from the remote device. 
+    This function code is used to get a status word, event count, message
+    count, and a field of event bytes from the remote device.
 
-    The status word and event counts are identical  to that returned by the
-    Get Communications Event Counter function (11, 0B hex). 
+    The status word and event counts are identical  to that returned by
+    the Get Communications Event Counter function (11, 0B hex).
 
     The message counter contains the quantity of  messages processed by the
-    remote device since its last restart, clear counters operation, or power-up.
-    This count is identical to that returned by the Diagnostic function
-    (code 08), sub-function Return Bus Message Count (code 11, 0B hex). 
+    remote device since its last restart, clear counters operation, or
+    power-up.  This count is identical to that returned by the Diagnostic
+    function (code 08), sub-function Return Bus Message Count (code 11,
+    0B hex).
 
-    The event bytes field contains 0-64 bytes, with each byte corresponding to
-    the status of one MODBUS send or receive operation for the remote device.
-    The remote device enters the events into the field in chronological order.
-    Byte 0 is the most recent event. Each new byte flushes the oldest byte
-    from the field.
+    The event bytes field contains 0-64 bytes, with each byte corresponding
+    to the status of one MODBUS send or receive operation for the remote
+    device.  The remote device enters the events into the field in
+    chronological order.  Byte 0 is the most recent event. Each new byte
+    flushes the oldest byte from the field.
     '''
     function_code = 0x0c
     _rtu_frame_size = 4
@@ -261,12 +268,13 @@ class GetCommEventLogRequest(ModbusRequest):
         '''
         return "GetCommEventLogRequest(%d)" % self.function_code
 
+
 class GetCommEventLogResponse(ModbusResponse):
     '''
     The normal response contains a two-byte status word field,
     a two-byte event count field, a two-byte message count field,
-    and a field containing 0-64 bytes of events. A byte count field 
-    defines the total length of the data in these four field
+    and a field containing 0-64 bytes of events. A byte count
+    field defines the total length of the data in these four field
     '''
     function_code = 0x0c
     _rtu_byte_count_pos = 3
@@ -290,7 +298,8 @@ class GetCommEventLogResponse(ModbusResponse):
 
         :returns: The byte encoded message
         '''
-        ready = ModbusStatus.Ready if self.status else ModbusStatus.Waiting
+        if self.status: ready = ModbusStatus.Ready
+        else: ready = ModbusStatus.Waiting
         packet  = struct.pack('>B', 6 + len(self.events))
         packet += struct.pack('>H', ready)
         packet += struct.pack('>HH', self.event_count, self.message_count)
@@ -309,7 +318,7 @@ class GetCommEventLogResponse(ModbusResponse):
         self.message_count = struct.unpack('>H', data[5:7])[0]
 
         self.events = []
-        for e in xrange(7, length+1):
+        for e in xrange(7, length + 1):
             self.events.append(struct.unpack('>B', data[e])[0])
 
     def __str__(self):
@@ -320,13 +329,14 @@ class GetCommEventLogResponse(ModbusResponse):
         arguments = (self.function_code, self.status, self.message_count, self.event_count)
         return "GetCommEventLogResponse(%d, %d, %d, %d)" % arguments
 
+
 #---------------------------------------------------------------------------#
 # TODO Make these only work on serial
 #---------------------------------------------------------------------------#
 class ReportSlaveIdRequest(ModbusRequest):
     '''
-    This function code is used to read the description of the type, the current
-    status, and other information specific to a remote device. 
+    This function code is used to read the description of the type, the
+    current status, and other information specific to a remote device.
     '''
     function_code = 0x11
     _rtu_frame_size = 4
@@ -363,15 +373,16 @@ class ReportSlaveIdRequest(ModbusRequest):
         '''
         return "ResportSlaveIdRequest(%d)" % self.function_code
 
+
 class ReportSlaveIdResponse(ModbusResponse):
     '''
-    The format of a normal response is shown in the following example. The
-    data contents are specific to each type of device.
+    The format of a normal response is shown in the following example.
+    The data contents are specific to each type of device.
     '''
     function_code = 0x11
     _rtu_byte_count_pos = 2
 
-    def __init__(self, identifier, status=True):
+    def __init__(self, identifier=0x00, status=True):
         ''' Initializes a new instance
 
         :param identifier: The identifier of the slave
@@ -386,11 +397,12 @@ class ReportSlaveIdResponse(ModbusResponse):
 
         :returns: The byte encoded message
         '''
-        status = ModbusStatus.SlaveOn if self.status else ModbusStatus.SlaveOff
+        if self.status: status = ModbusStatus.SlaveOn
+        else: status = ModbusStatus.SlaveOff
         length = len(self.identifier) + 2
         packet = struct.pack('>B', length)
-        packet += self.identifier # we assume it is already encoded
-        packet += struct.pack('>B', self.status)
+        packet += self.identifier  # we assume it is already encoded
+        packet += struct.pack('>B', status)
         return packet
 
     def decode(self, data):
@@ -402,7 +414,7 @@ class ReportSlaveIdResponse(ModbusResponse):
         :param data: The packet data to decode
         '''
         length = struct.unpack('>B', data[0])[0]
-        self.identifier = data[1:length-1]
+        self.identifier = data[1:length + 1]
         status = struct.unpack('>B', data[-1])[0]
         self.status = status == ModbusStatus.SlaveOn
 
@@ -419,9 +431,9 @@ class ReportSlaveIdResponse(ModbusResponse):
 #---------------------------------------------------------------------------#
 # report device identification 43, 14
 
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 # Exported symbols
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 __all__ = [
     "ReadExceptionStatusRequest", "ReadExceptionStatusResponse",
     "GetCommEventCounterRequest", "GetCommEventCounterResponse",
