@@ -7,6 +7,7 @@ from pymodbus.pdu import ModbusRequest
 from pymodbus.pdu import ModbusResponse
 from pymodbus.pdu import ModbusExceptions as merror
 
+
 class ReadRegistersRequestBase(ModbusRequest):
     '''
     Base class for reading a modbus register
@@ -44,6 +45,7 @@ class ReadRegistersRequestBase(ModbusRequest):
         '''
         return "ReadRegisterRequest (%d,%d)" % (self.address, self.count)
 
+
 class ReadRegistersResponseBase(ModbusResponse):
     '''
     Base class for responsing to a modbus register read
@@ -57,7 +59,7 @@ class ReadRegistersResponseBase(ModbusResponse):
         :param values: The values to write to
         '''
         ModbusResponse.__init__(self, **kwargs)
-        self.registers = values if values != None else []
+        self.registers = values or []
 
     def encode(self):
         ''' Encodes the response packet
@@ -77,7 +79,7 @@ class ReadRegistersResponseBase(ModbusResponse):
         byte_count = ord(data[0])
         self.registers = []
         for i in range(1, byte_count + 1, 2):
-            self.registers.append(struct.unpack('>H', data[i:i+2])[0])
+            self.registers.append(struct.unpack('>H', data[i:i + 2])[0])
 
     def getRegister(self, index):
         ''' Get the requested register
@@ -126,6 +128,7 @@ class ReadHoldingRegistersRequest(ReadRegistersRequestBase):
         values = context.getValues(self.function_code, self.address, self.count)
         return ReadHoldingRegistersResponse(values)
 
+
 class ReadHoldingRegistersResponse(ReadRegistersResponseBase):
     '''
     This function code is used to read the contents of a contiguous block
@@ -143,6 +146,7 @@ class ReadHoldingRegistersResponse(ReadRegistersResponseBase):
         '''
         ReadRegistersResponseBase.__init__(self, values, **kwargs)
 
+
 class ReadInputRegistersRequest(ReadRegistersRequestBase):
     '''
     This function code is used to read from 1 to approx. 125 contiguous
@@ -174,6 +178,7 @@ class ReadInputRegistersRequest(ReadRegistersRequestBase):
         values = context.getValues(self.function_code, self.address, self.count)
         return ReadInputRegistersResponse(values)
 
+
 class ReadInputRegistersResponse(ReadRegistersResponseBase):
     '''
     This function code is used to read from 1 to approx. 125 contiguous
@@ -191,6 +196,7 @@ class ReadInputRegistersResponse(ReadRegistersResponseBase):
         '''
         ReadRegistersResponseBase.__init__(self, values, **kwargs)
 
+
 class ReadWriteMultipleRegistersRequest(ModbusRequest):
     '''
     This function code performs a combination of one read operation and one
@@ -206,11 +212,9 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
     number of bytes to follow in the write data field."
     '''
     function_code = 23
-
     _rtu_byte_count_pos = 10
 
-    def __init__(self, read_address, read_count,
-        write_address, write_registers, **kwargs):
+    def __init__(self, **kwargs):
         ''' Initializes a new request message
 
         :param read_address: The address to start reading from
@@ -219,12 +223,12 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
         :param write_registers: The registers to write to the specified address
         '''
         ModbusRequest.__init__(self, **kwargs)
-        self.read_address  = read_address
-        self.read_count    = read_count
-        self.write_address = write_address
-        self.write_registers = write_registers
-        if not hasattr(write_registers, '__iter__'):
-            self.write_registers = [write_registers]
+        self.read_address    = kwargs.get('read_address', 0x00)
+        self.read_count      = kwargs.get('read_count', 0)
+        self.write_address   = kwargs.get('write_address', 0x00)
+        self.write_registers = kwargs.get('write_registers', None)
+        if not hasattr(self.write_registers, '__iter__'):
+            self.write_registers = [self.write_registers]
         self.write_count = len(self.write_registers)
         self.write_byte_count = self.write_count * 2
 
@@ -245,12 +249,12 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
 
         :param data: The request to decode
         '''
-        self.read_address, self.read_count, \
+        self.read_address,  self.read_count,  \
         self.write_address, self.write_count, \
         self.write_byte_count = struct.unpack('>HHHHB', data[:9])
-        self.write_registers = []
+        self.write_registers  = []
         for i in range(9, self.write_byte_count + 9, 2):
-            register = struct.unpack('>H', data[i:i+2])[0]
+            register = struct.unpack('>H', data[i:i + 2])[0]
             self.write_registers.append(register)
 
     def execute(self, context):
@@ -286,6 +290,7 @@ class ReadWriteMultipleRegistersRequest(ModbusRequest):
                   self.write_count)
         return "ReadWriteNRegisterRequest R(%d,%d) W(%d,%d)" % params
 
+
 class ReadWriteMultipleRegistersResponse(ModbusResponse):
     '''
     The normal response contains the data from the group of registers that
@@ -294,21 +299,21 @@ class ReadWriteMultipleRegistersResponse(ModbusResponse):
     '''
     function_code = 23
     _rtu_byte_count_pos = 2
-        
+
     def __init__(self, values=None, **kwargs):
         ''' Initializes a new instance
 
         :param values: The register values to write
         '''
         ModbusResponse.__init__(self, **kwargs)
-        self.registers = values if values != None else []
+        self.registers = values or []
 
     def encode(self):
         ''' Encodes the response packet
 
         :returns: The encoded packet
         '''
-        result = chr(len(self.registers)*2)
+        result = chr(len(self.registers) * 2)
         for register in self.registers:
             result += struct.pack('>H', register)
         return result
@@ -320,7 +325,7 @@ class ReadWriteMultipleRegistersResponse(ModbusResponse):
         '''
         bytes = ord(data[0])
         for i in range(1, bytes, 2):
-            self.registers.append(struct.unpack('>H', data[i:i+2])[0])
+            self.registers.append(struct.unpack('>H', data[i:i + 2])[0])
 
     def __str__(self):
         ''' Returns a string representation of the instance
@@ -329,9 +334,9 @@ class ReadWriteMultipleRegistersResponse(ModbusResponse):
         '''
         return "ReadWriteNRegisterResponse (%d)" % len(self.registers)
 
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 # Exported symbols
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 __all__ = [
     "ReadHoldingRegistersRequest", "ReadHoldingRegistersResponse",
     "ReadInputRegistersRequest", "ReadInputRegistersResponse",
