@@ -2,6 +2,7 @@
 Collection of transaction based abstractions
 '''
 import struct
+import socket
 from binascii import b2a_hex, a2b_hex
 
 from pymodbus.exceptions import ModbusIOException
@@ -16,10 +17,11 @@ from pymodbus.utilities  import checkLRC, computeLRC
 import logging
 _logger = logging.getLogger(__name__)
 
+
 #---------------------------------------------------------------------------#
 # The Global Transaction Manager
 #---------------------------------------------------------------------------#
-class ModbusTransactionManager(Singleton):
+class ModbusTransactionManager(object):
     ''' Impelements a transaction for a manager
 
     The transaction protocol can be represented by the following pseudo code::
@@ -31,44 +33,54 @@ class ModbusTransactionManager(Singleton):
              count++
           else break
         while (count < 3)
-    
+
     This module helps to abstract this away from the framer and protocol.
     '''
 
     __tid = Defaults.TransactionId
-    __transactions = []
+    __transactions = {}
 
-    def __init__(self):
-        ''' Initializes an instance of the ModbusTransactionManager '''
-        pass
+    def __init__(self, client=None):
+        ''' Initializes an instance of the ModbusTransactionManager
+
+        :param client: The client socket wrapper
+        '''
+        self.client = client
 
     def execute(self, request):
         ''' Starts the producer to send the next request to
         consumer.write(Frame(request))
         '''
         retries = Defaults.Retries
-        request.transaction_id = self.__getNextTID()
-        _logging.debug("Running transaction %d" % request.transaction_id)
+        request.transaction_id = self.getNextTID()
+        _logger.debug("Running transaction %d" % request.transaction_id)
 
         while retries > 0:
             try:
-                self.socket.connect()
-                packet = self.framer.buildPacket(request)
-                self.socket.send(packet)
+                self.client.connect()
+                self.client._send(self.client.framer.buildPacket(request))
+                # I need to fix this to read the header and the result size,
+                # as this may not read the full result set, but right now
+                # it should be fine...
+                result = self.client._recv(1024)
+                self.client.framer.processIncomingPacket(result, self.addTransaction)
+                break;
             except socket.error, msg:
-                self.socket.close()
-                _logging.debug("Transaction failed. (%s) " % msg)
+                self.client.close()
+                _logger.debug("Transaction failed. (%s) " % msg)
                 retries -= 1
+        return self.getTransaction(request.transaction_id)
 
     def addTransaction(self, request):
         ''' Adds a transaction to the handler
-           
+
         This holds the requets in case it needs to be resent.
         After being sent, the request is removed.
 
         :param request: The request to hold on to
         '''
-        ModbusTransactionManager.__transactions.append(request)
+        tid = request.transaction_id
+        ModbusTransactionManager.__transactions[tid] = request
 
     def getTransaction(self, tid):
         ''' Returns a transaction matching the referenced tid
@@ -77,23 +89,18 @@ class ModbusTransactionManager(Singleton):
 
         :param tid: The transaction to retrieve
         '''
-        for k,v in enumerate(ModbusTransactionManager.__transactions):
-            if v.transaction_id == tid:
-                return ModbusTransactionManager.__transactions.pop(k)
-        return None
+        return ModbusTransactionManager.__transactions.pop(tid, None)
 
     def delTransaction(self, tid):
         ''' Removes a transaction matching the referenced tid
 
         :param tid: The transaction to remove
         '''
-        for k,v in enumerate(ModbusTransactionManager.__transactions):
-            if v.transaction_id == tid:
-                del ModbusTransactionManager.__transactions[k]
+        ModbusTransactionManager.__transactions.pop(tid, None)
 
     def getNextTID(self):
         ''' Retrieve the next unique transaction identifier
-        
+
         This handles incrementing the identifier after
         retrieval
 
@@ -107,6 +114,7 @@ class ModbusTransactionManager(Singleton):
         ''' Resets the transaction identifier '''
         ModbusTransactionManager.__tid = Defaults.TransactionId
 
+
 #---------------------------------------------------------------------------#
 # Modbus TCP Message
 #---------------------------------------------------------------------------#
@@ -115,16 +123,16 @@ class ModbusSocketFramer(IModbusFramer):
 
     Before each modbus TCP message is an MBAP header which is used as a
     message frame.  It allows us to easily separate messages as follows::
-    
+
         [         MBAP Header         ] [ Function Code] [ Data ]
         [ tid ][ pid ][ length ][ uid ]
           2b     2b     2b        1b           1b           Nb
-        
+
         while len(message) > 0:
             tid, pid, length`, uid = struct.unpack(">HHHB", message)
             request = message[0:7 + length - 1`]
             message = [7 + length - 1:]
-        
+
         * length = uid + function code + data
         * The -1 is to account for the uid byte
     '''
@@ -232,7 +240,7 @@ class ModbusSocketFramer(IModbusFramer):
                     raise ModbusIOException("Unable to decode request")
                 self.populateResult(result)
                 self.advanceFrame()
-                callback(result) # defer or push to a thread?
+                callback(result)  # defer or push to a thread?
             else: break
 
     def buildPacket(self, message):
@@ -249,6 +257,7 @@ class ModbusSocketFramer(IModbusFramer):
             message.function_code) + data
         return packet
 
+
 #---------------------------------------------------------------------------#
 # Modbus RTU Message
 #---------------------------------------------------------------------------#
@@ -258,13 +267,13 @@ class ModbusRtuFramer(IModbusFramer):
 
         [ Start Wait ] [Address ][ Function Code] [ Data ][ CRC ][  End Wait  ]
           3.5 chars     1b         1b               Nb      2b      3.5 chars
-        
+
     Wait refers to the amount of time required to transmist at least x many
     characters.  In this case it is 3.5 characters.  Also, if we recieve a
     wait of 1.5 characters at any point, we must trigger an error message.
     Also, it appears as though this message is little endian. The logic is
     simplified as the following::
-        
+
         block-on-read:
             read until 3.5 delay
             check for errors
@@ -272,7 +281,7 @@ class ModbusRtuFramer(IModbusFramer):
 
     The following table is a listing of the baud wait times for the specified
     baud rates::
-        
+
         ------------------------------------------------------------------
          Baud  1.5c (18 bits)   3.5c (38 bits)
         ------------------------------------------------------------------
@@ -336,7 +345,7 @@ class ModbusRtuFramer(IModbusFramer):
 
     def populateHeader(self):
         ''' Try to set the headers `uid`, `len` and `crc`.
-        
+
         This method examines `self.__buffer` and writes meta
         information into `self.__header`. It calculates only the
         values for headers that are not already in the dictionary.
@@ -352,7 +361,7 @@ class ModbusRtuFramer(IModbusFramer):
             size = pdu_class.calculateRtuFrameSize(self.__buffer)
             self.__header['len'] = size
         if 'crc' not in self.__header:
-            self.__header['crc'] = self.__buffer[size-2:size]
+            self.__header['crc'] = self.__buffer[size - 2:size]
 
     def addToFrame(self, message):
         '''
@@ -371,11 +380,12 @@ class ModbusRtuFramer(IModbusFramer):
         start  = self.__hsize
         end    = self.__header['len'] - 2
         buffer = self.__buffer[start:end]
-        return buffer if end > 0 else ''
+        if end > 0: return buffer
+        return ''
 
     def populateResult(self, result):
         ''' Populates the modbus result header
-   
+
         The serial packets do not have any header information
         that is copied.
 
@@ -409,7 +419,7 @@ class ModbusRtuFramer(IModbusFramer):
                     raise ModbusIOException("Unable to decode response")
                 self.populateResult(result)
                 self.advanceFrame()
-                callback(result) # defer or push to a thread?
+                callback(result)  # defer or push to a thread?
             else: break
 
     def buildPacket(self, message):
@@ -424,21 +434,22 @@ class ModbusRtuFramer(IModbusFramer):
         packet += struct.pack(">H", computeCRC(packet))
         return packet
 
+
 #---------------------------------------------------------------------------#
 # Modbus ASCII Message
 #---------------------------------------------------------------------------#
 class ModbusAsciiFramer(IModbusFramer):
     '''
     Modbus ASCII Frame Controller::
-        
+
         [ Start ][Address ][ Function ][ Data ][ LRC ][ End ]
           1c        2c         2c         Nc     2c      2c
-        
+
         * data can be 0 - 2x252 chars
         * end is '\\r\\n' (Carriage return line feed), however the line feed
           character can be changed via a special command
         * start is ':'
-   
+
     This framer is used for serial transmission.  Unlike the RTU protocol,
     the data in this framer is transferred in plain text ascii.
     '''
@@ -465,17 +476,17 @@ class ModbusAsciiFramer(IModbusFramer):
         '''
         start = self.__buffer.find(self.__start)
         if start == -1: return False
-        if start > 0 : # go ahead and skip old bad data
+        if start > 0 :  # go ahead and skip old bad data
             self.__buffer = self.__buffer[start:]
+            start = 0
 
         end = self.__buffer.find(self.__end)
         if (end != -1):
             self.__header['len'] = end
             self.__header['uid'] = int(self.__buffer[1:3], 16)
-            self.__header['lrc'] = int(self.__buffer[end-2:end], 16)
-            #data = self.__buffer[start:end-2]
-            #return checkLRC(data, self.__header['lrc'])
-            return True
+            self.__header['lrc'] = int(self.__buffer[end - 2:end], 16)
+            data = a2b_hex(self.__buffer[start + 1:end - 2])
+            return checkLRC(data, self.__header['lrc'])
         return False
 
     def advanceFrame(self):
@@ -513,11 +524,12 @@ class ModbusAsciiFramer(IModbusFramer):
         start  = self.__hsize + 1
         end    = self.__header['len'] - 2
         buffer = self.__buffer[start:end]
-        return a2b_hex(buffer) if end > 0 else ''
+        if end > 0: return a2b_hex(buffer)
+        return ''
 
     def populateResult(self, result):
         ''' Populates the modbus result header
-   
+
         The serial packets do not have any header information
         that is copied.
 
@@ -551,7 +563,7 @@ class ModbusAsciiFramer(IModbusFramer):
                     raise ModbusIOException("Unable to decode response")
                 self.populateResult(result)
                 self.advanceFrame()
-                callback(result) # defer this
+                callback(result)  # defer this
             else: break
 
     def buildPacket(self, message):
@@ -570,6 +582,7 @@ class ModbusAsciiFramer(IModbusFramer):
         packet = '%c%s%02x%s' % (self.__start, packet, checksum, self.__end)
         return packet.upper()
 
+
 #---------------------------------------------------------------------------#
 # Modbus Binary Message
 #---------------------------------------------------------------------------#
@@ -606,8 +619,8 @@ class ModbusBinaryFramer(IModbusFramer):
         self.__buffer = ''
         self.__header = {'crc':0x0000, 'len':0, 'uid':0x00}
         self.__hsize  = 0x02
-        self.__start  = '\x7b' # {
-        self.__end    = '\x7d' # }
+        self.__start  = '\x7b'  # {
+        self.__end    = '\x7d'  # }
         self.decoder  = decoder
 
     #-----------------------------------------------------------------------#
@@ -620,18 +633,16 @@ class ModbusBinaryFramer(IModbusFramer):
         '''
         start = self.__buffer.find(self.__start)
         if start == -1: return False
-        if start > 0 : # go ahead and skip old bad data
+        if start > 0 :  # go ahead and skip old bad data
             self.__buffer = self.__buffer[start:]
 
         end = self.__buffer.find(self.__end)
         if (end != -1):
             self.__header['len'] = end
             self.__header['uid'] = struct.unpack('>B', self.__buffer[1:2])
-            self.__header['crc'] = self.__buffer[end-2:end]
-            #self.__header['crc'] = struct.unpack('>H', self.__buffer[end-2:end])
-            #data = self.__buffer[start:end-1]
-            #return checkCRC(data, self.__header['crc'])
-            return True
+            self.__header['crc'] = struct.unpack('>H', self.__buffer[end - 2:end])[0]
+            data = self.__buffer[start:end - 2]
+            return checkCRC(data, self.__header['crc'])
         return False
 
     def advanceFrame(self):
@@ -669,11 +680,12 @@ class ModbusBinaryFramer(IModbusFramer):
         start  = self.__hsize + 1
         end    = self.__header['len'] - 2
         buffer = self.__buffer[start:end]
-        return buffer if end > 0 else ''
+        if end > 0: return buffer
+        return ''
 
     def populateResult(self, result):
         ''' Populates the modbus result header
-   
+
         The serial packets do not have any header information
         that is copied.
 
@@ -707,7 +719,7 @@ class ModbusBinaryFramer(IModbusFramer):
                     raise ModbusIOException("Unable to decode response")
                 self.populateResult(result)
                 self.advanceFrame()
-                callback(result) # defer or push to a thread?
+                callback(result)  # defer or push to a thread?
             else: break
 
     def buildPacket(self, message):
@@ -721,6 +733,7 @@ class ModbusBinaryFramer(IModbusFramer):
             message.unit_id,
             message.function_code) + data
         packet += struct.pack(">H", computeCRC(packet))
+        packet = '%s%s%s' % (self.__start, packet, self.__end)
         return packet
 
     def _preflight(self, data):
@@ -733,12 +746,13 @@ class ModbusBinaryFramer(IModbusFramer):
         :returns: the escaped packet
         '''
         def _filter(a):
-            return a*2 if a in ['}', '{'] else a, data
+            if a in ['}', '{']: return a * 2
+            else: return a
         return ''.join(map(_filter, data))
 
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 # Exported symbols
-#---------------------------------------------------------------------------# 
+#---------------------------------------------------------------------------#
 __all__ = [
     "ModbusTransactionManager",
     "ModbusSocketFramer", "ModbusRtuFramer",
